// This sample monitors the user's browsing behavior, and looks for web pages that 
// are from amazon.com. When it finds such a page, it uses a regular expression that locates the title 
// and author of the book. It then does a query for the book title and author followed by 
// the terms "inurl:book review"
// Thus it tries to show web pages that are book reviews of the book the user is viewing
// on Amazon. The MatcherQueryBuilder class can be extended to form any such query,
// and hence that's why we call this plugin the "I'm Feeling Lucky," because it depends
// on useful Google Search results for its utility.

// Setup the icons, about dialog stuff, and flags to be nice.
// the str* variables are taken from the strings.js file in the appropriate locale directory
pluginHelper.title = strTitle;
pluginHelper.SetIcons(utils.loadImage("plugin_small.gif"), utils.loadImage("plugin_large.gif"));
pluginHelper.about_text = strAboutText;
pluginHelper.SetFlags(gddPluginFlagNone, gddContentFlagHaveDetails);

// initialize the plugin
var plugin = new ImFeelingLuckyPlugin();

// this class does a Google web query using Google's SOAP API.
// see http://api.google.com/ to understand the XML output of the results
function GoogleWebQuery(search_terms, num_results, callbackFunction) {
  // TODO(developer): change the XX key, and get your own key at http://www.google.com/apis/
  // (start at step 2 "Create a Google Account")
  var search_key = 'XX';
  if (search_key == 'XX') {
    utils.log(gddLogLevelError, '', 'No search key provided. Cannot execute Google query.');
    utils.alert('You need to get a Google Search key to use this sample. Visit http://www.google.com/apis/ to get a key.');
    return;
  }
  var xmlhttp = new XMLHttpRequest();  
  // create a SOAP request to get Google results
  xmlhttp.open("POST", "http://api.google.com/search/beta2", true);
  xmlhttp.onreadystatechange= OnData;
  xmlhttp.setRequestHeader("Man", "POST http://api.google.com/search/beta2 HTTP/1.1");
  xmlhttp.setRequestHeader("MessageType", "CALL");
  xmlhttp.setRequestHeader("Content-Type", "text/xml");
  xmlhttp.send("<?xml version='1.0' encoding='UTF-8'?>"+"\n\n"+"<SOAP-ENV:Envelope"+
        ' xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"'+
        ' xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance"'+
        ' xmlns:xsd="http://www.w3.org/1999/XMLSchema">'+
        '<SOAP-ENV:Body><ns1:doGoogleSearch'+
        ' xmlns:ns1="urn:GoogleSearch"'+
        ' SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">'+
        '<key xsi:type="xsd:string">' + search_key + '</key> <q'+
        ' xsi:type="xsd:string">'+search_terms+'</q> <start'+
        ' xsi:type="xsd:int">0</start> <maxResults'+
        ' xsi:type="xsd:int">'+ parseInt(num_results) + '</maxResults> <filter'+
        ' xsi:type="xsd:boolean">true</filter> <restrict'+
        ' xsi:type="xsd:string"></restrict> <safeSearch'+
        ' xsi:type="xsd:boolean">false</safeSearch> <lr'+
        ' xsi:type="xsd:string"></lr> <ie'+
        ' xsi:type="xsd:string">latin1</ie> <oe'+
        ' xsi:type="xsd:string">latin1</oe>'+
        '</ns1:doGoogleSearch>'+
      '</SOAP-ENV:Body></SOAP-ENV:Envelope>');

  // we call the callback with the results returned from Google
  function OnData() {
    if (xmlhttp.readyState == 4) { // completed?
      callbackFunction(xmlhttp.responseText);
    }
  } 
}

// this class takes a regular expression for a url. if a web page the user is viewing
// matches this regular express (url_regexp_str), then a second regular expression
// is used to extract strings from the content of the page (page_regexp_str). 
// The matches within this regular expressions are then inserted into the query_template
// (match 1 goes into the label {1} in the query template, match 2 goes in at {2}, etc..)
// and the query is executed on Google and results are returned  
function MatcherQueryBuilder(url_regexp_str, page_regexp_str, query_template) {
  this.url_regexp = new RegExp(url_regexp_str, 'i');
  this.page_regexp = new RegExp(page_regexp_str, 'i');
  
  this.query_template = query_template;
  this.url_regexp_str = url_regexp_str;
  this.page_regexp_str = page_regexp_str;
  var delimiter = '####';
  
  this.isUrlMatch = function(url) {
    return this.url_regexp.test(url);
  }
  
  this.getQuery = function(content) {
    var matches = this.page_regexp.exec(content);
    if (matches != null) {
      var query = this.query_template;
      // we start at 1 because matches[0] is the full string, which we don't about
      for (var i = 1; i < matches.length; ++i) {
        query = query.replace("{"+parseInt(i)+"}", matches[i]);
      }
      utils.log(gddLogLevelInfo, "", "executing query: " + query)
      return query;
    }
  }
  
  this.toString = function() {
    var as_string = this.url_regexp_str + delimiter + this.page_regexp_str + delimiter + this.query_template;
    return as_string;
  }
  
  this.fromString = function(as_string) {
    var my_array = as_string.split(delimiter);
    this.url_regexp = new RegExp(my_array[0], 'i');
    this.page_regexp = new RegExp(my_array[1], 'i');
    this.query_template = my_array[2];
  }
  
}

function ImFeelingLuckyPlugin() {
  var matchers = new Array();
  // load up our matchers from options, which is a global variable provided by the sidebar
  if (options.GetValue("matchers_0") == "") {
    utils.log(gddLogLevelInfo, "", "no old matchers found. starting with amazon.com matcher")
    matchers = new Array(
      // this default matcher looks at amazon pages, finds the book and author name, and generates a query looking for a book review
      new MatcherQueryBuilder('amazon', '<title>Amazon.com: (.+): Books(: (.+))*</title>', '"{1}" "{3}" inurl:book review'));
    saveMatchers();
  } else {
    utils.log(gddLogLevelInfo, "", "loading old matcher data")  
    loadMatchers();
  }
  
  // initialize our connection to the Google Desktop Event Api
  initializeEventApi();
  
  function loadMatchers() {
    var data = " ";
    for (var i = 0; data != ""; ++i) {
      data = options.GetValue("matchers_" + i);
      if (data != "") {
        // construct a Matcher and then load it from its string representation
        var m = new MatcherQueryBuilder("", "", "")
        m.fromString(data);
        matchers.push(m);
      }
    }
  }

  function saveMatchers() {
    options.Clear();
    for (var i = 0; i < matchers.length; ++i)
      options.PutValue("matchers_" + i, matchers[i].toString());
  }

  function initializeEventApi() {
    try {
      var subscription = eventPublisher.Subscribe(onNewEvent);
      var coll = subscription.AddFilter("GoogleDesktop.FilterCollection");
      coll.filter_operator = 1; // 0 = and, 1 = or
      var schema = coll.AddFilter("GoogleDesktop.SchemaFilter");

      // Configure the schema filter to only show webpages
      // other schemas are listed in the Google Desktop documentation
      schema.Allow("Google.Desktop.WebPage");

      // Turn on the event stream
      subscription.active = true;
    } catch(e) {
      utils.log(gddLogLevelError, "", "could not hook into Event API. error: " + e.description);
      utils.alert("init: " + e.description);
    }
  }

  function onNewEvent(event_object) {
    if (event_object != null) {
      var url = event_object.GetProperty("uri");
      for (var i = 0; i < matchers.length; ++i) {      
        if (matchers[i].isUrlMatch(url)) {
          var content = event_object.GetProperty("content");
          var query = matchers[i].getQuery(content);
          if (query) 
            ImFeelingLuckyItem(query, url);
        }
      }
    }
  }
}

// this class creates and pushes to the sidebar a content item based on an I'm Feeling Lucky
// type search to Google
function ImFeelingLuckyItem(search_terms, source_url) {  
  var source_domain = GetDomainName(source_url);
  var google_query = new GoogleWebQuery(search_terms, 2, OnData);
  function OnData(result_text) {
    var doc = new DOMDocument();
    doc.loadXML(result_text);
    var elem = doc.getElementsByTagName("item");
    for (var i = 0; i < elem.length ; i = i + 1) {
      var url, snippet, title;
      for (var node = elem[0].firstChild; node != null; node = node.nextSibling) {
        if (node.nodeName == "URL")
          url = node.firstChild.nodeValue;
        else if (node.nodeName == "snippet")
          snippet = RemoveTags(node.firstChild.nodeValue);
        else if (node.nodeName == "title")
          title = RemoveTags(node.firstChild.nodeValue);
      }
      // Since this is an "I'm Feeling Lucky" search, we show only the top
      // result, so break after the first valid result
      if (GetDomainName(url) != source_domain)
        break;
    }
    // we don't want to show results from the same website that originally 
    // induced the query (seems sort of pointless)
    if (url != null && GetDomainName(url) != source_domain) {
      var curItem = new ContentItem();
      curItem.source = GetDomainName(url);
      curItem.layout = gddContentItemLayoutNews;
      curItem.onDetailsView = OnDetailsView;
      pluginHelper.AddContentItem(curItem, gddItemDisplayInSidebar);
      
      var heading = title + ":\n" + snippet;
      curItem.heading = heading;
      curItem.snippet = snippet;     // to show in details view
      curItem.open_command = url;
    }
  }

  function OnDetailsView(item) {
    var obj = new Object();
    obj.title = item.heading.substring(0, (item.heading.indexOf(":")));
    // other properties we can set in this object are 'cancel', 'details_control' and
    // 'flags'.
    return obj;
  }
}

// ----------- Utility functions ---------------

function RemoveTags(item) {
  item = item.replace(/<([^>]|\n)*>/g,'');  // remove html tags if present
  item = item.replace(/&nbsp;/g, ' ');
  item = item.replace(/&quot;/g, '"');
  item = item.replace(/&amp;/g, '&');
  item = item.replace(/&lt;/g, '<');
  item = item.replace(/&gt;/g, '>');
  return item;
}

function GetDomainName(url) {
  var i;
  if ((i = url.indexOf("//")) != -1)
    url = url.substring(i + 2);
  if ((i = url.indexOf("/")) != -1)
    url = url.substring(0, i);
  if ((i = url.indexOf("www.")) == 0)
    url = url.substring(4, url.length);
  return url;
}